******************************************************************
* COPYRIGHT (C) 1986 by Donald Krantz and James Stanley
* - Note: This is a real, live, actual, registered copyright,
*   and should be treated as such. This source code is from
*   the book "68000 Assembly Language", Krantz and Stanley,
*   Addison-Wesley Publishing Company, Reading, MA, 1986.
* 
*   Permission granted by the authors for non-commercial use
*   in programs released to the public domain, as long as this
*   copyright notice remains attached and visible.
*
*****************************************************************
* WINDOW.ASM - General purpose window support
*
* OPEN_W: Opens a screen window (borders & clears).
*	ENTRY: A5.L points to an initialized edit descriptor
*
* CLEAR_W: Clears the text area of a window.
*	ENTRY: A5.L points to an initialized edit descriptor
*
* BORDER_W: Draws a window border.
*	ENTRY: A5.L points to an initialized edit descriptor
*
* SET_W: Initializes screen parameters in a window descriptor
*	ENTRY: A5.L points to an edit descriptor
*		D0.W = upper left corner X position
*		D1.W = upper left corner Y position
*		D2.W = lower right corner X position
*		D3.W = lower right corner Y position
*
*****************************************************************
* Character equates for window border characters - IBM-PC
ul_cor	equ	201		* Upper left window border char
ur_cor	equ	187		* Upper right window border char
ll_cor	equ	200		* Lower left window border char
lr_cor	equ	188		* Lower right window border char
horizn	equ	205		* Horizontal window border char
vertic	equ	186		* Vertical window border char
*****************************************************************
#edit.h
*****************************************************************
	xdef	open_w,border_w,clear_w,set_w
	xref	cursor,putc
*****************************************************************
*  OPEN_W - Opens a screen window.
open_w:
	bsr	border_w	* make window border
	bsr	clear_w		* erase window
	rts
*****************************************************************
* BORDER_W - borders a window.
border_w:
	movem.l	d0-d2,-(a7)	* save caller's registers
	bsr	do_top		* draw top of window
	bsr	do_mid		* draw sides and text area
	bsr	do_bot		* draw bottom of window
	movem.l	(a7)+,d0-d2	* restore caller's registers
	rts
do_top:
	move.w	w_ulby(a5),-(a7) * push up/left border Y position
	move.w	w_ulbx(a5),-(a7) * push up/left border X position
	bsr	cursor		* set cursor
	addq.l	#4,a7		* pop parameter
	move.w	#ul_cor,-(a7)	* push up/left border char
	bsr	putc		* output the char
	addq.l	#2,a7		* pop parameter
	move.w	w_cols(a5),d2	* get column count
	move.w	#horizn,-(a7)	* push border char - only once
	subq.w	#1,d2		* adjust for loop index
lp0_dt:
	bsr	putc		* output horizontal border char
	dbra	d2,lp0_dt	* loop
	move.w	#ur_cor,(a7)	* "Push" up/right border char
	bsr	putc		* output it
	addq.l	#2,a7		* pop parameter
	rts			* finished
do_mid:
	clr.w	d1		* setup line counter	
lp3_dt:	
	addq.w	#1,d1		* increment line counter
	move.w	w_ulby(a5),-(a7) * push up/left border Y position
	add.w	d1,(a7)		* add current line number
	move.w	w_ulbx(a5),-(a7) * push left border X position
	bsr	cursor		* set cursor
	addq.l	#4,a7		* pop parameters
	move.w	#vertic,-(a7)	* push vertical border char
	bsr	putc		* output the char
	addq.l	#2,a7		* pop parameter
	move.w	w_ulby(a5),-(a7) * push up/left border Y position
	add.w	d1,(a7)		* add current line number
	move.w	w_lrbx(a5),-(a7) * push right border X position
	bsr	cursor		* set cursor
	addq.l	#4,a7		* pop parameters
	move.w	#vertic,-(a7)	* push vertical border char
	bsr	putc		* output the char
	addq.l	#2,a7		* pop parameter
	cmp.w	w_rows(a5),d1	* enough?
	ble	lp3_dt		* nope, need more lines.
	rts			* finished
do_bot:
	move.w	w_lrby(a5),-(a7) * push low/left border Y position
	move.w	w_ulbx(a5),-(a7) * push low/left border X position
	bsr	cursor		* set cursor
	addq.l	#4,a7		* pop parameter
	move.w	#ll_cor,-(a7)	* push low/left border char
	bsr	putc		* output the char
	addq.l	#2,a7		* pop parameter
	move.w	w_cols(a5),d2	* get column count
	move.w	#horizn,-(a7)	* push border char - only once
	subq.w	#1,d2		* adjust for loop index
lp2_dt:
	bsr	putc		* output horizontal border char
	dbra	d2,lp2_dt	* loop
	move.w	#lr_cor,(a7)	* "Push" low/right border char
	bsr	putc		* output it
	addq.l	#2,a7		* pop parameter
	rts			* finished
*****************************************************************
* CLEAR_W - Clears a window to blanks.
clear_w:
	movem.l	d0-d2,-(a7)	* save callers registers
	clr.w	d1		* setup line counter	
lp1_cw:	
	addq.w	#1,d1		* increment line counter
	move.w	w_ulby(a5),-(a7) * push up/left border Y position
	add.w	d1,(a7)		* add current line number
	move.w	w_ulcx(a5),-(a7) * push left char X position
	bsr	cursor		* set cursor
	addq.l	#4,a7		* pop parameters
	move.w	w_cols(a5),d2	* get column count
	move.w	#'  ',-(a7)	* push background char - only once
	subq.w	#1,d2		* adjust for loop index
lp2_cw:
	bsr	putc		* output background char
	dbra	d2,lp2_cw	* loop
	addq.l	#2,a7		* pop parameter
	cmp.w	w_rows(a5),d1	* enough?
	blt	lp1_cw		* nope, need more lines.
	movem.l	(a7)+,d0-d2	* restore callers registers
	rts			* finished
*****************************************************************
* SET_W: Initializes screen parameters in a window descriptor
set_w:
	movem.w	d0-d3,-(a7)	* Save registers
	move.w	d0,w_ulbx(a5)	* upper left border X
	move.w	d1,w_ulby(a5)	* upper left border Y
	move.w	d2,w_lrbx(a5)	* lower right border X
	move.w	d3,w_lrby(a5)	* lower right border Y
	subq.w	#1,d2
	move.w	d2,w_lrcx(a5)	* lower right character X
	subq.w	#1,d3
	move.w	d3,w_lrcy(a5)	* lower right character Y
	sub.w	d0,d2		* calculates usable columns
	move.w	d2,w_cols(a5)	* window columns
	sub.w	d1,d3		* calculates usable rows
	move.w	d3,w_rows(a5)	* window rows
	addq.w	#1,d0		
	move.w	d0,w_ulcx(a5)	* upper left character X
	addq.w	#1,d1	
	move.w	d1,w_ulcy(a5)	* upper left character Y
	movem.w	(a7)+,d0-d3	* Restore registers
	rts

	end
